@@ -7,6 +7,7 @@
 #include <sqlite3.h>
 
 #include "database.h"
+#include "log.h"
 
 #define count(x) (sizeof((x))/sizeof(*(x)))
 #define check(r) if((r) != SQLITE_OK) { goto sqlerror; }
@@ -17,7 +18,8 @@ static sqlite3_uint64 gettime(void)
     struct timespec now;
     if(clock_gettime(CLOCK_MONOTONIC, &now) == -1)
     {
-        perror("Getting time failed (clock_gettime)");
+        log_critical_(0, "getting time failed (clock_gettime): ");
+        perror(NULL);
         exit(1);
     }
     timestamp = now.tv_sec;
@@ -67,7 +69,7 @@ int db_init(const char *filename)
         else
         {
         wrongschema:
-            fprintf(stderr, "Database schema is wrong\n");
+            log_critical(0, "database schema is wrong");
             return -1;
         }
         sqlite3_finalize(stmt_get_tables);
@@ -89,6 +91,7 @@ int db_init(const char *filename)
             "    name TEXT NOT NULL,"
             "    timestamp INTEGER NOT NULL,"
             "    mode INTEGER NOT NULL,"
+            "    is_directory BOOLEAN NOT NULL,"
             "    process INTEGER NOT NULL"
             "    );",
             "CREATE TABLE executed_files("
@@ -128,8 +131,9 @@ int db_init(const char *filename)
 
     {
         const char *sql = ""
-                "INSERT INTO opened_files(name, timestamp, mode, process)"
-                "VALUES(?, ?, ?, ?)";
+                "INSERT INTO opened_files(name, timestamp, "
+                "        mode, is_directory, process)"
+                "VALUES(?, ?, ?, ?, ?)";
         check(sqlite3_prepare_v2(db, sql, -1, &stmt_insert_file, NULL));
     }
 
@@ -144,8 +148,7 @@ int db_init(const char *filename)
     return 0;
 
 sqlerror:
-    fprintf(stderr, "sqlite3 error creating database: %s\n",
-            sqlite3_errmsg(db));
+    log_critical(0, "sqlite3 error creating database: %s", sqlite3_errmsg(db));
     return -1;
 }
 
@@ -160,7 +163,7 @@ int db_close(void)
     return 0;
 
 sqlerror:
-    fprintf(stderr, "sqlite3 error on exit: %s\n", sqlite3_errmsg(db));
+    log_critical(0, "sqlite3 error on exit: %s", sqlite3_errmsg(db));
     return -1;
 }
 
@@ -192,11 +195,10 @@ int db_add_process(unsigned int *id, unsigned int parent_id,
         goto sqlerror;
     sqlite3_reset(stmt_last_rowid);
 
-    return db_add_file_open(*id, working_dir, FILE_WDIR);
+    return db_add_file_open(*id, working_dir, FILE_WDIR, 1);
 
 sqlerror:
-    fprintf(stderr, "sqlite3 error inserting process: %s\n",
-            sqlite3_errmsg(db));
+    log_critical(0, "sqlite3 error inserting process: %s", sqlite3_errmsg(db));
     return -1;
 }
 
@@ -217,18 +219,19 @@ int db_add_exit(unsigned int id, int exitcode)
     return 0;
 
 sqlerror:
-    fprintf(stderr, "sqlite3 error setting exitcode: %s\n",
-            sqlite3_errmsg(db));
+    log_critical(0, "sqlite3 error setting exitcode: %s", sqlite3_errmsg(db));
     return -1;
 }
 
-int db_add_file_open(unsigned int process, const char *name, unsigned int mode)
+int db_add_file_open(unsigned int process, const char *name,
+                     unsigned int mode, int is_dir)
 {
     check(sqlite3_bind_text(stmt_insert_file, 1, name, -1, SQLITE_TRANSIENT));
     /* This assumes that we won't go over 2^32 seconds (~135 years) */
     check(sqlite3_bind_int64(stmt_insert_file, 2, gettime()));
     check(sqlite3_bind_int(stmt_insert_file, 3, mode));
-    check(sqlite3_bind_int(stmt_insert_file, 4, process));
+    check(sqlite3_bind_int(stmt_insert_file, 4, is_dir));
+    check(sqlite3_bind_int(stmt_insert_file, 5, process));
 
     if(sqlite3_step(stmt_insert_file) != SQLITE_DONE)
         goto sqlerror;
@@ -236,8 +239,7 @@ int db_add_file_open(unsigned int process, const char *name, unsigned int mode)
     return 0;
 
 sqlerror:
-    fprintf(stderr, "sqlite3 error inserting file: %s\n",
-            sqlite3_errmsg(db));
+    log_critical(0, "sqlite3 error inserting file: %s", sqlite3_errmsg(db));
     return -1;
 }
 
@@ -302,7 +304,6 @@ int db_add_exec(unsigned int process, const char *binary,
     return 0;
 
 sqlerror:
-    fprintf(stderr, "sqlite3 error inserting exec: %s\n",
-            sqlite3_errmsg(db));
+    log_critical(0, "sqlite3 error inserting exec: %s", sqlite3_errmsg(db));
     return -1;
 }
