@@ -1,3 +1,4 @@
+#include <errno.h>
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
@@ -6,20 +7,44 @@
 #include <unistd.h>
 
 #include "config.h"
+#include "log.h"
 #include "ptrace_utils.h"
+#include "tracer.h"
 
 
-static void *tracee_getptr(pid_t pid, const void *addr)
+static long tracee_getword(pid_t tid, const void *addr)
 {
-    return (void*)ptrace(PTRACE_PEEKDATA, pid, addr, NULL);
+    long res;
+    errno = 0;
+    res = ptrace(PTRACE_PEEKDATA, tid, addr, NULL);
+    if(errno)
+    {
+        log_error_(tid, "tracee_getword() failed: ");
+        perror(NULL);
+        return 0;
+    }
+    return res;
 }
 
-static long tracee_getword(pid_t pid, const void *addr)
+static void *tracee_getptr(int mode, pid_t tid, const void *addr)
 {
-    return ptrace(PTRACE_PEEKDATA, pid, addr, NULL);
+    if(mode == MODE_I386)
+    {
+        /* Pointers are 32 bits */
+        uint32_t ptr;
+        tracee_read(tid, (void*)&ptr, addr, sizeof(ptr));
+        return (void*)(uint64_t)ptr;
+    }
+    else /* mode == MODE_X86_64 */
+    {
+        /* Pointers are 64 bits */
+        uint64_t ptr;
+        tracee_read(tid, (void*)&ptr, addr, sizeof(ptr));
+        return (void*)ptr;
+    }
 }
 
-size_t tracee_strlen(pid_t pid, const char *str)
+size_t tracee_strlen(pid_t tid, const char *str)
 {
     uintptr_t ptr = (uintptr_t)str;
     size_t j = ptr % WORD_SIZE;
@@ -28,7 +53,7 @@ size_t tracee_strlen(pid_t pid, const char *str)
     int done = 0;
     for(; !done; i += WORD_SIZE)
     {
-        unsigned long data = tracee_getword(pid, (const void*)i);
+        unsigned long data = tracee_getword(tid, (const void*)i);
         for(; !done && j < WORD_SIZE; ++j)
         {
             unsigned char byte = data >> (8 * j);
@@ -42,7 +67,7 @@ size_t tracee_strlen(pid_t pid, const char *str)
     return size;
 }
 
-void tracee_read(pid_t pid, char *dst, const char *src, size_t size)
+void tracee_read(pid_t tid, char *dst, const char *src, size_t size)
 {
     uintptr_t ptr = (uintptr_t)src;
     size_t j = ptr % WORD_SIZE;
@@ -50,36 +75,37 @@ void tracee_read(pid_t pid, char *dst, const char *src, size_t size)
     uintptr_t end = ptr + size;
     for(; i < end; i += WORD_SIZE)
     {
-        unsigned long data = tracee_getword(pid, (const void*)i);
+        unsigned long data = tracee_getword(tid, (const void*)i);
         for(; j < WORD_SIZE && i + j < end; ++j)
             *dst++ = data >> (8 * j);
         j = 0;
     }
 }
 
-char *tracee_strdup(pid_t pid, const char *str)
+char *tracee_strdup(pid_t tid, const char *str)
 {
-    size_t length = tracee_strlen(pid, str);
+    size_t length = tracee_strlen(tid, str);
     char *res = malloc(length + 1);
-    tracee_read(pid, res, str, length);
+    tracee_read(tid, res, str, length);
     res[length] = '\0';
     return res;
 }
 
-char **tracee_strarraydup(pid_t pid, const char *const *argv)
+char **tracee_strarraydup(int mode, pid_t tid, const char *const *argv)
 {
+    /* FIXME : This is probably broken on x32 */
     char **array;
     /* Reads number of pointers in pointer array */
     size_t nb_args = 0;
     {
         const char *const *a = argv;
         /* xargv = *a */
-        const char *xargv = tracee_getptr(pid, a);
+        const char *xargv = tracee_getptr(mode, tid, a);
         while(xargv != NULL)
         {
             ++nb_args;
             ++a;
-            xargv = tracee_getptr(pid, a);
+            xargv = tracee_getptr(mode, tid, a);
         }
     }
     /* Allocs pointer array */
@@ -88,13 +114,13 @@ char **tracee_strarraydup(pid_t pid, const char *const *argv)
     {
         size_t i = 0;
         /* xargv = argv[0] */
-        const char *xargv = tracee_getptr(pid, argv);
+        const char *xargv = tracee_getptr(mode, tid, argv);
         while(xargv != NULL)
         {
-            array[i] = tracee_strdup(pid, xargv);
+            array[i] = tracee_strdup(tid, xargv);
             ++i;
             /* xargv = argv[i] */
-            xargv = tracee_getptr(pid, argv + i);
+            xargv = tracee_getptr(mode, tid, argv + i);
         }
         array[i] = NULL;
     }
